@@ -31,15 +31,7 @@
 
 using namespace mongocxx;
 
-std::string get_server_version(const client& client) {
-    bsoncxx::builder::basic::document server_status{};
-    server_status.append(bsoncxx::builder::basic::kvp("serverStatus", 1));
-    bsoncxx::document::value output = client["test"].run_command(server_status.extract());
-
-    return bsoncxx::string::to_string(output.view()["version"].get_string().value);
-}
-
-void aggregation_examples(const mongocxx::client& client, const mongocxx::database& db) {
+void aggregation_examples(const mongocxx::database& db) {
     {
         // Start Aggregation Example 1
         using namespace bsoncxx::builder::basic;
@@ -144,16 +136,9 @@ void aggregation_examples(const mongocxx::client& client, const mongocxx::databa
         auto cursor = db["air_alliances"].aggregate(p, mongocxx::options::aggregate{});
         // End Aggregation Example 4
 
-        // TODO: Remove version-check (CXX-1547)
-        // This example uses syntax added in mongo version 3.6. Our CI
-        // tooling runs tests with mongo version 3.4, so this example
-        // fails with a query exception. CXX-1547 tracks removing this
-        // version check once CI tooling uses a later version of the server.
-        if (get_server_version(client) >= "3.6") {
-            auto count = std::distance(cursor.begin(), cursor.end());
-            if (count != 0L) {
-                throw std::logic_error("wrong count in example 4");
-            }
+        auto count = std::distance(cursor.begin(), cursor.end());
+        if (count != 0L) {
+            throw std::logic_error("wrong count in example 4");
         }
     }
 }
@@ -167,8 +152,11 @@ int main() {
     const mongocxx::client conn{mongocxx::uri{}};
     auto const db = conn["documentation_examples"];
 
+    // SERVER-79306: Ensure the database exists for consistent behavior with sharded clusters.
+    conn["documentation_examples"].create_collection("dummy");
+
     try {
-        aggregation_examples(conn, db);
+        aggregation_examples(db);
     } catch (const std::logic_error& e) {
         std::cerr << e.what() << std::endl;
         return EXIT_FAILURE;
