@@ -14,10 +14,9 @@
 
 #include <set>
 
-#include "helpers.hpp"
 #include <bsoncxx/builder/basic/document.hpp>
 #include <bsoncxx/string/to_string.hpp>
-#include <bsoncxx/test_util/catch.hh>
+#include <bsoncxx/test/catch.hh>
 #include <mongocxx/client.hpp>
 #include <mongocxx/database.hpp>
 #include <mongocxx/exception/logic_error.hpp>
@@ -28,7 +27,8 @@
 #include <mongocxx/private/conversions.hh>
 #include <mongocxx/private/libbson.hh>
 #include <mongocxx/private/libmongoc.hh>
-#include <mongocxx/test_util/client_helpers.hh>
+#include <mongocxx/test/client_helpers.hh>
+#include <third_party/catch/include/helpers.hpp>
 
 namespace {
 using namespace mongocxx;
@@ -344,6 +344,10 @@ TEST_CASE("Database integration tests", "[database]") {
                 return;
             }
 
+            // SERVER-79306: $listLocalSessions does not behave as expect if the database does not
+            // already exist on sharded clusters.
+            database.create_collection("dummy");
+
             auto session1 = mongo_client.start_session();
 
             pipeline.list_local_sessions({});
