@@ -12,11 +12,11 @@
 // See the License for the specific language governing permissions and
 // limitations under the License.
 
-#include "helpers.hpp"
 #include <bsoncxx/builder/basic/document.hpp>
-#include <bsoncxx/test_util/catch.hh>
+#include <bsoncxx/test/catch.hh>
 #include <mongocxx/instance.hpp>
 #include <mongocxx/model/replace_one.hpp>
+#include <third_party/catch/include/helpers.hpp>
 
 namespace {
 using namespace bsoncxx::builder::basic;
@@ -38,4 +38,4 @@ TEST_CASE("replace_one model tests", "[replace_one][model]") {
     CHECK_OPTIONAL_ARGUMENT(ro, upsert, true);
     CHECK_OPTIONAL_ARGUMENT(ro, collation, collation.view());
 }
-}  // namespace
\ No newline at end of file
+}  // namespace
