@@ -14,11 +14,11 @@
 
 #include <chrono>
 
-#include "helpers.hpp"
 #include <bsoncxx/builder/basic/document.hpp>
-#include <bsoncxx/test_util/catch.hh>
+#include <bsoncxx/test/catch.hh>
 #include <mongocxx/instance.hpp>
 #include <mongocxx/options/count.hpp>
+#include <third_party/catch/include/helpers.hpp>
 
 namespace {
 using namespace bsoncxx::builder::basic;
