@@ -29,7 +29,7 @@
 #include <bsoncxx/stdx/make_unique.hpp>
 #include <bsoncxx/stdx/string_view.hpp>
 #include <bsoncxx/string/to_string.hpp>
-#include <bsoncxx/test_util/catch.hh>
+#include <bsoncxx/test/catch.hh>
 #include <bsoncxx/types/bson_value/view.hpp>
 #include <mongocxx/client.hpp>
 #include <mongocxx/collection.hpp>
@@ -40,8 +40,8 @@
 #include <mongocxx/instance.hpp>
 #include <mongocxx/options/gridfs/upload.hpp>
 #include <mongocxx/result/gridfs/upload.hpp>
+#include <mongocxx/test/client_helpers.hh>
 #include <mongocxx/test/spec/util.hh>
-#include <mongocxx/test_util/client_helpers.hh>
 
 namespace {
 using namespace bsoncxx;
