@@ -18,7 +18,7 @@
 #include <bsoncxx/builder/basic/document.hpp>
 #include <bsoncxx/json.hpp>
 #include <bsoncxx/string/to_string.hpp>
-#include <bsoncxx/test_util/catch.hh>
+#include <bsoncxx/test/catch.hh>
 #include <bsoncxx/types.hpp>
 #include <mongocxx/client.hpp>
 #include <mongocxx/collection.hpp>
@@ -45,7 +45,7 @@
 #include <mongocxx/config/private/prelude.hh>
 
 namespace mongocxx {
-MONGOCXX_INLINE_NAMESPACE_BEGIN
+inline namespace v_noabi {
 namespace spec {
 
 using namespace mongocxx;
@@ -125,7 +125,9 @@ bsoncxx::stdx::optional<read_preference> lookup_read_preference(document::view d
 client_session* operation_runner::_lookup_session(stdx::string_view key) {
     if (key.compare("session0") == 0) {
         return _session0;
-    } else {
+    }
+
+    if (key.compare("session1") == 0) {
         return _session1;
     }
 
@@ -134,8 +136,7 @@ client_session* operation_runner::_lookup_session(stdx::string_view key) {
 
 client_session* operation_runner::_lookup_session(document::view doc) {
     if (doc["session"]) {
-        stdx::string_view session_name = doc["session"].get_string().value;
-        return _lookup_session(session_name);
+        return _lookup_session(doc["session"].get_string().value);
     }
     return nullptr;
 }
@@ -1243,22 +1244,6 @@ document::value operation_runner::_run_run_command(bsoncxx::document::view opera
     return result.extract();
 }
 
-document::value operation_runner::_run_configure_fail_point(bsoncxx::document::view operation) {
-    auto arguments = operation["arguments"].get_document().value;
-    auto command = arguments["failPoint"].get_document().value;
-
-    const client_session* session = _lookup_session(arguments);
-
-    read_preference rp;
-    uint32_t server_id = session->server_id();
-    stdx::optional<document::value> reply = (*_client)["admin"].run_command(command, server_id);
-
-    auto result = builder::basic::document{};
-    result.append(builder::basic::kvp("result", *reply));
-
-    return result.extract();
-}
-
 document::value operation_runner::_create_index(const document::view& operation) {
     auto arguments = operation["arguments"];
     auto session = _lookup_session(arguments.get_document().value);
@@ -1378,17 +1363,40 @@ document::value operation_runner::run(document::view operation) {
         return _run_abort_transaction(operation);
     } else if (key.compare("runCommand") == 0) {
         return _run_run_command(operation);
-    } else if (key.compare("targetedFailPoint") == 0) {
-        return _run_configure_fail_point(operation);
     } else if (key.compare("assertSessionPinned") == 0) {
         const client_session* session =
             _lookup_session(operation["arguments"].get_document().value);
-        REQUIRE(session->server_id());
+        REQUIRE(session);
+        REQUIRE(session->server_id() != 0);
         return empty_document;
-    } else if (key.compare("operationassertSessionUnpinned") == 0) {
+    } else if (key.compare("assertSessionUnpinned") == 0) {
         const client_session* session =
             _lookup_session(operation["arguments"].get_document().value);
-        REQUIRE(!session->server_id());
+        REQUIRE(session);
+        REQUIRE(session->server_id() == 0);
+        return empty_document;
+    } else if (key.compare("assertSessionTransactionState") == 0) {
+        const auto arguments = operation["arguments"].get_document().value;
+        const client_session* session = _lookup_session(arguments);
+        REQUIRE(session);
+        const auto state = arguments["state"].get_string().value;
+        switch (session->get_transaction_state()) {
+            case client_session::transaction_state::k_transaction_none:
+                REQUIRE(state == stdx::string_view("none"));
+                break;
+            case client_session::transaction_state::k_transaction_starting:
+                REQUIRE(state == stdx::string_view("starting"));
+                break;
+            case client_session::transaction_state::k_transaction_in_progress:
+                REQUIRE(state == stdx::string_view("in_progress"));
+                break;
+            case client_session::transaction_state::k_transaction_committed:
+                REQUIRE(state == stdx::string_view("committed"));
+                break;
+            case client_session::transaction_state::k_transaction_aborted:
+                REQUIRE(state == stdx::string_view("aborted"));
+                break;
+        }
         return empty_document;
     } else if (key.compare("watch") == 0) {
         if (object.compare("collection") == 0) {
@@ -1473,6 +1481,26 @@ document::value operation_runner::run(document::view operation) {
                                          operation["arguments"]["index"].get_string());
                              }));
 
+        return empty_document;
+    } else if (key.compare("targetedFailPoint") == 0) {
+        REQUIRE(object == stdx::string_view("testRunner"));
+
+        const auto arguments = operation["arguments"].get_document().value;
+
+        const auto session_ptr = _lookup_session(arguments);
+        REQUIRE(session_ptr);
+        auto& session = *session_ptr;
+        const auto server_id = session.server_id();
+
+        if (server_id == 0) {
+            FAIL("session object is not pinned to a mongos server");
+        }
+
+        const auto command = arguments["failPoint"].get_document().value;
+        REQUIRE(!command.empty());
+
+        session.client()["admin"].run_command(command, server_id);
+
         return empty_document;
     } else {
         throw std::logic_error{"unsupported operation: " + string::to_string(key)};
@@ -1480,7 +1508,7 @@ document::value operation_runner::run(document::view operation) {
 }
 
 }  // namespace spec
-MONGOCXX_INLINE_NAMESPACE_END
+}  // namespace v_noabi
 }  // namespace mongocxx
 
 #include <mongocxx/config/private/postlude.hh>
