@@ -12,7 +12,7 @@
 // See the License for the specific language governing permissions and
 // limitations under the License.
 
-#include <bsoncxx/test_util/catch.hh>
+#include <bsoncxx/test/catch.hh>
 #include <mongocxx/instance.hpp>
 #include <mongocxx/test/spec/util.hh>
 
@@ -24,12 +24,21 @@ using namespace spec;
 TEST_CASE("Transactions spec automated tests", "[transactions_spec]") {
     instance::current();
 
-    /* Tests that use operations that the C++ driver does not have. */
-    std::set<std::string> unsupported_transaction_tests = {"count.json"};
+    // Tests that use operations that the C++ driver does not have.
+    std::set<std::string> unsupported_transaction_tests = {
+        // C Driver does not support count helper.
+        "count.json",
+    };
 
-    run_tests_in_suite(
-        "TRANSACTIONS_TESTS_PATH", &run_transactions_tests_in_file, unsupported_transaction_tests);
+    SECTION("Legacy") {
+        run_tests_in_suite("TRANSACTIONS_LEGACY_TESTS_PATH",
+                           &run_transactions_tests_in_file,
+                           unsupported_transaction_tests);
+    }
 
-    run_tests_in_suite("WITH_TRANSACTION_TESTS_PATH", &run_transactions_tests_in_file);
+    SECTION("Convenient API") {
+        run_tests_in_suite("WITH_TRANSACTION_TESTS_PATH", &run_transactions_tests_in_file);
+    }
 }
+
 }  // namespace
