@@ -18,8 +18,10 @@
 
 #include <bsoncxx/string/to_string.hpp>
 
+#include <mongocxx/config/prelude.hpp>
+
 namespace mongocxx {
-MONGOCXX_INLINE_NAMESPACE_BEGIN
+inline namespace v_noabi {
 namespace entity {
 
 template <typename Key, typename Entity, typename Map>
@@ -151,5 +153,5 @@ void map::erase(const key_type& key) {
 }
 
 }  // namespace entity
-MONGOCXX_INLINE_NAMESPACE_END
+}  // namespace v_noabi
 }  // namespace mongocxx
