@@ -14,8 +14,8 @@
 
 #include "operations.hh"
 
-#include <catch.hpp>
 #include <sstream>
+#include <vector>
 
 #include <bsoncxx/document/value.hpp>
 #include <bsoncxx/json.hpp>
@@ -24,10 +24,12 @@
 #include <mongocxx/collection.hpp>
 #include <mongocxx/exception/logic_error.hpp>
 #include <mongocxx/test/spec/monitoring.hh>
+#include <third_party/catch/include/catch.hpp>
+
+#include <mongocxx/config/prelude.hpp>
 
 namespace mongocxx {
-MONGOCXX_INLINE_NAMESPACE_BEGIN
-
+inline namespace v_noabi {
 using namespace bsoncxx;
 
 using builder::basic::kvp;
@@ -1794,6 +1796,96 @@ document::value create_find_cursor(entity::map& map,
     return make_document();
 }
 
+document::value create_search_index(collection& coll, document::view operation) {
+    const auto arguments = operation["arguments"];
+    const auto raw_model = arguments["model"];
+    const auto name = raw_model["name"];
+    const auto definition = raw_model["definition"].get_document().value;
+
+    const auto model = name ? search_index_model(name.get_string().value, definition)
+                            : search_index_model(definition);
+
+    return make_document(kvp("result", coll.search_indexes().create_one(model)));
+}
+
+document::value create_search_indexes(collection& coll, document::view operation) {
+    auto arguments = operation["arguments"];
+
+    auto raw_models = arguments["models"].get_array().value;
+    std::vector<search_index_model> models;
+
+    for (auto&& m : raw_models) {
+        search_index_model model = m["name"]
+                                       ? search_index_model(m["name"].get_string().value,
+                                                            m["definition"].get_document().value)
+                                       : search_index_model(m["definition"].get_document().value);
+        models.push_back(model);
+    }
+
+    std::vector<std::string> created_indexes = coll.search_indexes().create_many(models);
+
+    builder::basic::array result;
+    for (auto s : created_indexes) {
+        result.append(s);
+    }
+
+    return make_document(kvp("result", result.view()));
+}
+
+document::value drop_search_index(collection& coll, document::view operation) {
+    auto arguments = operation["arguments"];
+
+    auto name = arguments["name"].get_string().value;
+
+    coll.search_indexes().drop_one(name);
+
+    // no return value from drop_one
+    return make_document();
+}
+
+document::value list_search_indexes(collection& coll, document::view operation) {
+    auto arguments = operation["arguments"];
+    options::aggregate options;
+    if (arguments["aggregationOptions"]) {
+        const auto aggregation_options = arguments["aggregationOptions"].get_document().view();
+        for (auto&& element : aggregation_options) {
+            if (element.key() == stdx::string_view("batchSize")) {
+                options.batch_size(element.get_int32().value);
+            } else {
+                throw std::logic_error{"unsupported aggregateOptions field: " +
+                                       std::string(element.key())};
+            }
+        }
+    }
+
+    cursor c = arguments["name"]
+                   ? coll.search_indexes().list(arguments["name"].get_string().value, options)
+                   : coll.search_indexes().list(options);
+
+    // we must loop over the resulting cursor to trigger server events for the spec tests
+    auto result = bsoncxx::builder::basic::document{};
+    result.append(
+        bsoncxx::builder::basic::kvp("result", [&c](bsoncxx::builder::basic::sub_array array) {
+            for (auto&& document : c) {
+                array.append(document);
+            }
+        }));
+
+    return result.extract();
+}
+
+document::value update_search_index(collection& coll, document::view operation) {
+    auto arguments = operation["arguments"];
+
+    auto name = arguments["name"].get_string().value;
+    auto definition = arguments["definition"].get_document().value;
+
+    coll.search_indexes().update_one(name, definition);
+
+    // no return value from update_one
+    return make_document();
+}
+
 document::value operations::run(entity::map& entity_map,
                                 std::unordered_map<std::string, spec::apm_checker>& apm_map,
                                 const array::element& op,
@@ -1871,6 +1963,36 @@ document::value operations::run(entity::map& entity_map,
         auto key = string::to_string(op["arguments"]["client"].get_string().value);
         return fail_point(entity_map, apm_map[key], op_view);
     }
+    if (name == "targetedFailPoint") {
+        REQUIRE(object == "testRunner");
+
+        const auto arguments = op_view["arguments"];
+
+        const auto fail_point = arguments["failPoint"].get_document().value;
+        REQUIRE(!fail_point.empty());
+
+        auto* const session_ptr = get_session(op_view, entity_map);
+        REQUIRE(session_ptr);
+        auto& session = *session_ptr;
+        const auto server_id = session.server_id();
+
+        // Test runners MUST error if the session is not pinned to a mongos server at the time
+        // this operation is executed.
+        if (server_id == 0) {
+            FAIL("session object is not pinned to a mongos server");
+        }
+
+        auto& client = session.client();
+
+        // If the driver exposes an API to target a specific server for a command, the test runner
+        // SHOULD use the client entity associated with the session to execute the
+        // configureFailPoint command.
+        client["admin"].run_command(fail_point, server_id);
+
+        return make_document(kvp("uri", client.uri().to_string()),
+                             kvp("failPoint", fail_point["configureFailPoint"].get_string()),
+                             kvp("serverId", static_cast<std::int64_t>(server_id)));
+    }
     if (name == "findOneAndDelete")
         return find_one_and_delete(
             entity_map.get_collection(object), get_session(op_view, entity_map), op_view);
@@ -1949,6 +2071,18 @@ document::value operations::run(entity::map& entity_map,
         auto& session = entity_map.get_client_session(session_name);
         return assert_session_transaction_state(session, op_view);
     }
+    if (name == "assertSessionPinned") {
+        auto& session = entity_map.get_client_session(
+            string::to_string(op["arguments"]["session"].get_string().value));
+        REQUIRE(session.server_id() != 0);
+        return make_document();
+    }
+    if (name == "assertSessionUnpinned") {
+        auto& session = entity_map.get_client_session(
+            string::to_string(op["arguments"]["session"].get_string().value));
+        REQUIRE(session.server_id() == 0);
+        return make_document();
+    }
     if (name == "dropCollection") {
         auto coll_name = string::to_string(op["arguments"]["collection"].get_string().value);
         auto& db = entity_map.get_database(object);
@@ -2129,9 +2263,34 @@ document::value operations::run(entity::map& entity_map,
     if (name == "removeKeyAltName") {
         return remove_key_alt_name(entity_map, object, op_view);
     }
+    if (name == "createSearchIndex") {
+        auto& coll = entity_map.get_collection(object);
+
+        return create_search_index(coll, op_view);
+    }
+    if (name == "createSearchIndexes") {
+        auto& coll = entity_map.get_collection(object);
+
+        return create_search_indexes(coll, op_view);
+    }
+    if (name == "dropSearchIndex") {
+        auto& coll = entity_map.get_collection(object);
+
+        return drop_search_index(coll, op_view);
+    }
+    if (name == "listSearchIndexes") {
+        auto& coll = entity_map.get_collection(object);
+
+        return list_search_indexes(coll, op_view);
+    }
+    if (name == "updateSearchIndex") {
+        auto& coll = entity_map.get_collection(object);
+
+        return update_search_index(coll, op_view);
+    }
 
     throw std::logic_error{"unsupported operation: " + name};
 }
 
-MONGOCXX_INLINE_NAMESPACE_END
+}  // namespace v_noabi
 }  // namespace mongocxx
